void build_int_set(json &data,unique_ptr<int_set_t> &p) {
  string type = data["type"];
#define CHECK(name) if (type == #name) p.reset(new name)
  // initialize to anything
  p.reset(new odd_t);
  // cout << "p " << p.get() << endl;
#ifdef HAS_ODD_T
  CHECK(odd_t);
#endif
  
#ifdef HAS_EVEN_T  
  CHECK(even_t);
#endif
#ifdef HAS_ALL_T
  CHECK(all_t);
#endif
#ifdef HAS_EMPTY_T
  CHECK(empty_t);
#endif
#ifdef HAS_POSITIVE_T
  CHECK(positive_t);
#endif
#ifdef HAS_STRIDE_T
  if (type=="stride_t") {
    unique_ptr<stride_t> s(new stride_t);
    s->start = data["start"];
    s->end = data["end"];
    s->step = data["step"];
    p = move(s);
  }
#endif
#ifdef HAS_VECTOR_T
  if (type=="vector_t") {
    unique_ptr<vector_t> s(new vector_t);
    vector<int> ww = data["w"];
    s->w = ww;
    p = move(s);
  }
#endif
#ifdef HAS_INTERSECTION_T
  if (type=="intersection_t") {
    unique_ptr<intersection_t> s(new intersection_t);
    unique_ptr<int_set_t> s1,s2;
    build_int_set(data["s1"],s1);
    build_int_set(data["s2"],s2);
    s->s1 = move(s1);
    s->s2 = move(s2);
    p = move(s);
  }
#endif
#ifdef HAS_UNION_T
  if (type=="union_t") {
    unique_ptr<union_t> s(new union_t);
    unique_ptr<int_set_t> s1,s2;
    build_int_set(data["s1"],s1);
    build_int_set(data["s2"],s2);
    s->s1 = move(s1);
    s->s2 = move(s2);
    p = move(s);
  }
#endif
#ifdef HAS_DIFFERENCE_T
  if (type=="difference_t") {
    unique_ptr<difference_t> s(new difference_t);
    unique_ptr<int_set_t> s1,s2;
    build_int_set(data["s1"],s1);
    build_int_set(data["s2"],s2);
    s->s1 = move(s1);
    s->s2 = move(s2);
    p = move(s);
  }
#endif
#ifdef HAS_COMPLEMENT_T
  if (type=="complement_t") {
    unique_ptr<complement_t> s(new complement_t);
    unique_ptr<int_set_t> s1;
    build_int_set(data["s1"],s1);
    s->s1 = move(s1);
    p = move(s);
  }
#endif
}

void aed::check_int_set(json &data,json &output) {
  vector<int> probe = data["probe"];
  unique_ptr<int_set_t> p;
  build_int_set(data,p);
  cout << "type: " << typeid(*p.get()).name() << endl;
  vector<int> out;
  for (auto x : probe) {
    // cout << "x " << x << " isin? " << p->isin(x) << endl;
    if (p->isin(x)) out.push_back(x);
  }
  output = out;
}

void aed::check_nilpot(json &data,json &output) {
  unique_ptr<perm_t> perm(new perm_t);
  if (data["type"]=="shift1_t") {
#ifdef HAS_SHIFT1_T    
    perm.reset(new shift1_t);
#endif
  } else if (data["type"]=="shiftm_t") {
#ifdef HAS_SHIFTM_T    
    unique_ptr<shiftm_t> p(new shiftm_t);
    p->m = data["m"];
    perm = move(p);
#endif
  } else if (data["type"]=="oddeven_t") {
#ifdef HAS_ODDEVEN_T
    perm.reset(new oddeven_t);
#endif    
  } else if (data["type"]=="reflex_t") {
#ifdef HAS_REFLEX_T
    perm.reset(new reflex_t);
#endif
  } else if (data["type"]=="shiftm1m2_t") {
#ifdef HAS_SHIFTM1M2_T
    unique_ptr<shiftm1m2_t> p(new shiftm1m2_t);
    p->m1 = data["m1"];
    p->m2 = data["m2"];
    perm = move(p);
#endif
  }
  // cout << "type: " << typeid(*perm.get()).name() << endl;
  int N = data["N"];
  vector<int> w(N);
  for (int j=0; j<N; j++) w[j]=j;
  perm->mix(w);
  output["mixed"] = w;
  output["np"] = nilpot(*perm.get(),N);
}
