#include <iostream>
#include <Eigen/Dense>

// Calcular la superficie de una esfera
using namespace Eigen;
using namespace std;

// Computes the are of a triangle in 3D
double area(Vector3d &y0,Vector3d &y1,Vector3d &y2) {
  Vector3d a=y1-y0, b=y2-y0,nor;
  nor = a.cross(b);
  return nor.norm()/2.0;
}

// Projects the position X on the ellipsoid of axis 1,1,c
// The shift rotates points in space (p,q,r) -> (r,p,q) etc...
void project(Vector3d &x,double c=1.0,int shift=0) {
  // Do the shift
  Vector3d tmp;
  int ndim=3;
  for (int j=0; j<ndim; j++)
    tmp(j) = x((j+shift)%ndim);
  x = tmp;
  // Normalize and scale the Z coordinate
  x /= x.norm(); x(2)*=c;
}

int main() {
  // N is the number of segments in each axis
  int N=200;
  // Step size
  double h=1.0/N;
  // Positions of the vertex of the quadrangle.
  // XC will be the center. 
  Vector3d x1,x2,x3,x4,xc;
  // Accumulator for the area
  // C is the radius in Z direction
  double Area=0.0,c=0.5,a=1.0;
  // Compute the areas for the projections of 1/4th of the
  // plane at z=1, x=1, and y=1. 
  for (int shift=0; shift<3; shift++) {
    // Loop over the quadrangles
    for (int j=0; j<N; j++) {
      double x=j*h;
      for (int k=0; k<N; k++) {
        double y=k*h;
        // X;Y is the corner of the quadrangle
        x1 << x,y,1.0;
        x2 << x+h,y,1.0;
        x3 << x+h,y+h,1.0;
        x4 << x,y+h,1.0;
        // shift and project on the ellipsoid
        project(x1,c,shift);
        project(x2,c,shift);
        project(x3,c,shift);
        project(x4,c,shift);
        // Compute the center of the quadrangle
        xc = (x1+x2+x3+x4)/4.0;
        // Compute the area of 4 triangles composed of each
        // side of the quadrangle and the center of the sphere
        Area += area(x1,x2,xc);
        Area += area(x2,x3,xc);
        Area += area(x3,x4,xc);
        Area += area(x4,x1,xc);
      }
    }
  }
  // The computed area is 1/8th of the total ellipsoid surface
  Area *= 8;
  // Use the exact value for comparison
  // https://en.wikipedia.org/wiki/Ellipsoid#Surface_area
  double Areaex,e;
  if (c<a) {
    e = sqrt(1-(c*c)/(a*a));
    Areaex = 2.0*M_PI*(1.0+(c*c)/(e*a*a)*atanh(e));
  } else {
    e = sqrt(1-(a*a)/(c+c));
    Areaex = 2.0*M_PI*(1.0+c/(a*e)*asin(e));
  }
  // Error 
  double erro = fabs(Area-Areaex);
  cout << "Area, Area(ex), error "
       << Area << " " << Areaex << " " << erro << endl;
  return 0;
}
